var prog, tex;
var cam_theta = 0;
var cam_phi = 0;
var cam_dist = 4.0;
var cam_x = 0, cam_y = 0, cam_z = 0;
var rotate_auto = true;

var seed = [0.4, 0.0, 0.0, -0.8];
var err_thres = 0.0075;
var iter = 10;
var reflectivity = 0.4;
//var reflectivity = 0.0;
var color = [0.38, 0.35, 0.32];
//var color = [0.8, 0.8, 0.8];
var show_floor = true;

var loc_aspect, loc_seed, loc_err, loc_iter, loc_diffuse, loc_refl;

function init_gl()
{
	glass_init("glcanvas", "console");
	glass_display_func(disp);
	glass_mouse_func(mouse);
	glass_motion_func(motion);
	glass_reshape_func(reshape);

	if(!(prog = load_program("rtj.v.glsl", "rtj.p.glsl"))) {
		return;
	}

	set_uniform1f(prog, "vfov", Math.PI / 4.0);
	set_uniform1i(prog, "show_floor", show_floor ? 1 : 0);

	loc_aspect = gl.getUniformLocation(prog, "aspect");
	loc_seed = gl.getUniformLocation(prog, "seed");
	loc_err = gl.getUniformLocation(prog, "err_thres");
	loc_iter = gl.getUniformLocation(prog, "iter");
	loc_refl = gl.getUniformLocation(prog, "reflectivity");
	loc_diffuse = gl.getUniformLocation(prog, "diffuse_color");

	glass_start();
}

function disp(msec)
{
	//gl.clear(gl.COLOR_BUFFER_BIT);

	if(rotate_auto) {
		cam_theta = msec / 200.0;
		cam_phi = 0;
	}

	seed[0] = Math.cos(msec / 3000.0);
	seed[3] = Math.sin(msec / 3000.0) * 1.25 - 0.25;

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(cam_x, cam_y, -cam_z);
	glRotatef(cam_theta, 0, 1, 0);
	glRotatef(cam_phi, 1, 0, 0);
	glTranslatef(0, 0, -cam_dist);

	gl.useProgram(prog);

	gl.uniform1f(loc_aspect, canvas.width / canvas.height);
	gl.uniform4fv(loc_seed, seed);
	gl.uniform1f(loc_err, err_thres);
	gl.uniform1i(loc_iter, iter);
	gl.uniform3fv(loc_diffuse, color);
	gl.uniform1f(loc_refl, reflectivity);

	gl.bindTexture(gl.TEXTURE_2D, tex);

	glBegin(GL_QUADS);
	glTexCoord2f(0, 0);
	glVertex2f(-1, -1);
	glTexCoord2f(1, 0);
	glVertex2f(1, -1);
	glTexCoord2f(1, 1);
	glVertex2f(1, 1);
	glTexCoord2f(0, 1);
	glVertex2f(-1, 1);
	glEnd();

	gl.bindTexture(gl.TEXTURE_2D, null);
}

function reshape(x, y)
{
	gl.viewport(0, 0, x, y);
}

function toggle_floor()
{
	show_floor = !show_floor;
	set_uniform1i(prog, "show_floor", show_floor ? 1 : 0);
}

var prev_x = -1, prev_y = -1;
var bnstate = [false, false, false];
var timeout;

function mouse(bn, state, x, y)
{
	bnstate[bn] = state;
	if(state) {
		prev_x = x;
		prev_y = y;

		clearTimeout(timeout);
		rotate_auto = false;
	} else {
		prev_x = prev_y = -1;

		timeout = setTimeout(function() { rotate_auto = true; }, 5000);
	}
}

function motion(x, y)
{
	if(prev_x != -1) {
		cam_theta += (x - prev_x) * 0.5;
		cam_phi += (y - prev_y) * 0.5;

		if(cam_phi < -90) cam_phi = -90;
		if(cam_phi > 90) cam_phi = 90;

		prev_x = x;
		prev_y = y;
	}
}
